//--------------------------------------------------------------
// File     : main.c
// Datum    : 21.09.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : CMSIS_BOOT, M4_CMSIS_CORE
// Funktion : 2D-Game per Tiled
// Hinweis  : Diese zwei Files muessen auf 8MHz stehen
//              "cmsis_boot/stm32f4xx.h"
//              "cmsis_boot/system_stm32f4xx.c"
// In Configuration diese Define hinzufgen :
// "STM32F429_439xx" , "__ASSEMBLY__" , "USE_STDPERIPH_DRIVER"
//--------------------------------------------------------------

#include "main.h"
#include "stm32_ub_font.h"
#include "stm32_ub_tiled.h"
#include "stm32_ub_usb_hid_host.h"
#include <stdio.h> // fuer printf

//--------------------------------------------------------------
// Globale Variablen fuer das Spiel
//--------------------------------------------------------------
USB_HID_HOST_STATUS_t usb_status;  // status vom Keyboard
uint32_t points=0;                 // punkte
int32_t health=100;                // leben
uint32_t mode=0;                   // 0=run, 1=lose, 2=win
char buf[30];

//--------------------------------------------------------------
// Hauptprogramm fuer das Spiel
//--------------------------------------------------------------
int main(void)
{
  ErrorStatus check;

  SystemInit(); // Quarz Einstellungen aktivieren

  // init vom USB-Port als HOST
  UB_USB_HID_HOST_Init();

  // init von Tiled (mit check der MAP-Daten)
  check=UB_Tiled_Init(true);
  if(check!=SUCCESS) {
    UB_Font_DrawString(10,10,"Tiled Error",&Arial_10x15,RGB_COL_RED,RGB_COL_WHITE);
    sprintf(buf,"Err.Nr.=%d ",(int)(TILED_STATUS.err_nr));
    UB_Font_DrawString(10,40,buf,&Arial_10x15,RGB_COL_RED,RGB_COL_WHITE);
    while(1);
  }

  while(1)
  {
    // Tiled Prozesse bearbeiten
    UB_Tiled_Do();

    // check ob Spiel zu ende ist (win oder lose)
    if(mode!=0) UB_Tiled_Stop(); // anhalten
  }
}


//--------------------------------------------------------------
// User-Routine von Tiled
// (1ms Interrupt)
//--------------------------------------------------------------
void UB_Tiled_1ms_ISR_CallBack(void)
{
  // USB-Keyboard bearbeiten
  usb_status=UB_USB_HID_HOST_Do();
}


//--------------------------------------------------------------
// User-Routine von Tiled
// (bearbeiten vom Eingabegert)
//--------------------------------------------------------------
void UB_Tiled_Input_Device_CallBack(void)
{
  if(usb_status==USB_HID_KEYBOARD_CONNECTED) {
    // check ob eine Taste gedrueckt ist
    if(UB_USB_HID_HOST_GetKeyAnz()>0) {
      // Bewegung vom Player per Curosr-Tasten
      if(USB_KEY_DATA.akt_key1==79) UB_Tiled_MovePlayer(DIR_LEFT,4);  // 79=cursor links
      if(USB_KEY_DATA.akt_key1==89) UB_Tiled_MovePlayer(DIR_RIGHT,4); // 89=cursor rechts
      if(USB_KEY_DATA.akt_key1==83) UB_Tiled_MovePlayer(DIR_UP,4);    // 83=cursor hoch
      if(USB_KEY_DATA.akt_key1==84) UB_Tiled_MovePlayer(DIR_DOWN,4);  // 84=cursor runter
    }
    else UB_Tiled_MovePlayer(DIR_NONE,0); // keine Taste gedrueckt
  }
}


//--------------------------------------------------------------
// User-Routine von Tiled
// (Display Anzeige aktuallisieren)
//--------------------------------------------------------------
void UB_Tiled_LCD_CallBack(void)
{
  uint32_t n=0;

  // Status vom Keyboard anzeigen
  if(usb_status==USB_HID_KEYBOARD_CONNECTED) {
    UB_Font_DrawString(10,270,"Keyboard OK   ",&Arial_10x15,RGB_COL_RED,RGB_COL_WHITE);
  }
  else {
    UB_Font_DrawString(10,270,"Keyboard ERROR",&Arial_10x15,RGB_COL_WHITE,RGB_COL_RED);
  }

  // Abgelaufene Spielzeit anzeigen
  n=TILED.game_time_sec;
  sprintf(buf,"T=%d ",(int)(n));
  UB_Font_DrawString(170,270,buf,&Arial_10x15,RGB_COL_WHITE,RGB_COL_BLACK);

  // Anzeige der Punkte
  sprintf(buf,"Points=%d ",(int)(points));
  UB_Font_DrawString(10,295,buf,&Arial_10x15,RGB_COL_WHITE,RGB_COL_BLACK);

  // Anzeige der Healh bzw dem Spielstand
  if(mode==0) {
    sprintf(buf,"Health=%d ",(int)(health));
    UB_Font_DrawString(120,295,buf,&Arial_10x15,RGB_COL_WHITE,RGB_COL_BLACK);
  }
  else if(mode==1) {
    UB_Font_DrawString(120,295,"YOU LOSE  ",&Arial_10x15,RGB_COL_WHITE,RGB_COL_RED);
  }
  else if(mode==2) {
    UB_Font_DrawString(120,295,"YOU WIN   ",&Arial_10x15,RGB_COL_BLACK,RGB_COL_GREEN);
  }
}


//--------------------------------------------------------------
// User-Routine von Tiled
// (Event ist aufgetreten -> "TILED.event")
//--------------------------------------------------------------
void UB_Tiled_Event_CallBack(void)
{
  uint32_t obj_nr=0;
  uint8_t obj_typ=0;

  // check auf Kollision mit Sammelbarem
  if(TILED.event==TILED_EVENT_COLLECTABLE_COLLISION) {
	// pruefen wer die Kollision verursacht hat
    UB_Tiled_CheckCollectableCollision(&obj_nr, &obj_typ);
    // Objekt deaktivieren = eingesammelt
    UB_Tiled_DisableObjekt(obj_nr);
    // je nach objekt entweder punkte oder health erhoehen
    if(obj_typ==TILED_OBJ_COIN) points+=15;
    if(obj_typ==TILED_OBJ_POISON1) health+=5;
    if(obj_typ==TILED_OBJ_POISON2) health+=10;
  }

  // check auf Kollision mit Objekt
  if(TILED.event==TILED_EVENT_DEADLY_COLLISION) {
    // pruefen wer die Kollision verursacht hat
    UB_Tiled_CheckDeadlyCollision(&obj_nr, &obj_typ);
    // // je nach objekt health verringern
    if(obj_typ==TILED_OBJ_KNIGHT1) health--;
    if(obj_typ==TILED_OBJ_MAGE1) health-=2;
    if(obj_typ==TILED_OBJ_MONSTER1) health-=4;
    if(obj_typ==TILED_OBJ_DEVIL1) health-=6;
    if(health<=0) mode=1;  // lose
  }

  // check auf Ziel
  if(TILED.event==TILED_EVENT_GOAL) {
    mode=2; // win
  }
}

