//--------------------------------------------------------------
// File     : stm32_ub_tiled_map.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_TILED_MAP_H
#define __STM32F4_UB_TILED_MAP_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32_ub_tiled_global.h"
#include "stm32_ub_graphic2d.h"
#include "stm32_ub_tiled_terrain.h"


//--------------------------------------------------------------
// Tileset der MAP [USER_M02]
// (abhngig vom BMP-File des MAP-Tilesets)
//--------------------------------------------------------------
#define   TILESET_MAP_NAME  Image_Map  // Name vom Tileset
#define   TILESET_MAP_ANZX          8  // Anzahl der Tiles (x)
#define   TILESET_MAP_ANZY          6  // Anzahl der Tiles (y)
#define   TILESET_MAP_ANZ          48  // Anzahl der Tiles gesammt
#define   TILESET_MAP_TILEWIDTH    32  // breite eines Tiles in Pixel
#define   TILESET_MAP_TILEHEIGHT   32  // hoehe eines Tiles in Pixel
#define   TILESET_MAP_SPACING       1  // spacing in Pixel


//--------------------------------------------------------------
// Einstellung der MAP [USER_M03]
// (abhaengig vom TXT-File der exportieren MAP)
//--------------------------------------------------------------
#define   TILED_MAP_ANZX          40  // Anzahl der Tiles (x)
#define   TILED_MAP_ANZY          35  // Anzahl der Tiles (y)


//--------------------------------------------------------------
// Einstellung vom Screen [USER_M04]
// (sichtbarer Bereich auf dem Display)
//--------------------------------------------------------------
#define   TILED_SCREEN_ANZX          7  // Anzahl der Tiles (x)
#define   TILED_SCREEN_ANZY          8  // Anzahl der Tiles (y)
#define   TILED_SCREEN_XPOS          8  // Start-Xpos auf dem LCD (in Pixel)
#define   TILED_SCREEN_YPOS          5  // Start-Ypos auf dem LCD (in Pixel)


//--------------------------------------------------------------
// Startpositionen der MAP [USER_M05]
// (nach PowerOn)
//--------------------------------------------------------------
#define   TILED_MAP_STARTX           4  // Starttile der MAP (x)
#define   TILED_MAP_STARTY          27  // Starttile der MAP (y)


//--------------------------------------------------------------
// Minimum Abstand vom Player zum Rand [USER_M06]
// (wird fuer MAP automove benutzt)
//--------------------------------------------------------------
#define   TILED_SCREEN_BORDER       64  // Abstand vom Rand (in Pixel)


//--------------------------------------------------------------
// Hintergrundfarbe vom Display [USER_M07]
//--------------------------------------------------------------
#define   TILED_BG_COLOR    RGB_COL_BLACK



//--------------------------------------------------------------
// USER : ab hier keine Aenderungen notwendig
//--------------------------------------------------------------



//--------------------------------------------------------------
// Image vom Tileset der MAP
//--------------------------------------------------------------
extern UB_Image TILESET_MAP_NAME;      // (auch C-File einbinden)

//--------------------------------------------------------------
// Berechnungen
//--------------------------------------------------------------
#define   TILED_MAP_MAXX    (TILED_MAP_ANZX*TILESET_MAP_TILEWIDTH)   // Breit der Map in Pixel
#define   TILED_MAP_MAXY    (TILED_MAP_ANZY*TILESET_MAP_TILEHEIGHT)  // Hoehe der Map in Pixel
#define   TILED_MAP_ANZ     (TILED_MAP_ANZX*TILED_MAP_ANZY)          // Anzahl aller Tiles der MAP
//--------------------------------------------------------------
#define   TILED_SCREEN_MAXX     (TILED_SCREEN_ANZX*TILESET_MAP_TILEWIDTH)  // Breite vom Screen in Pixel
#define   TILED_SCREEN_MAXY     (TILED_SCREEN_ANZY*TILESET_MAP_TILEHEIGHT) // Hoehe vom Screen in Pixel
//--------------------------------------------------------------


//--------------------------------------------------------------
// Struktur : MAP
//--------------------------------------------------------------
typedef struct {
  uint32_t px_offsetx;  // offset als Pixelposition
  uint32_t px_offsety;  // offset als Pixelposition
}TILED_MAP_t;
TILED_MAP_t TILED_MAP;


//--------------------------------------------------------------
// Struktur : Border (wird errechnet)
// Border = welcher Bereich (in Pixel) der MAP sichtbar ist
//--------------------------------------------------------------
typedef struct {
  uint32_t px_startx;   // Start-Pixel X-Richtung (0...n)
  uint32_t px_endx;     // End-Pixel X-Richtung (0...n)
  uint32_t px_starty;   // Start-Pixel Y-Richtung (0...n)
  uint32_t px_endy;     // End-Pixel Y-Richtung (0...n)
}TILED_BORDER_t;
TILED_BORDER_t TILED_BORDER;


//--------------------------------------------------------------
// Struktur : TERRAIN
// Hinweis : jedes Tile ist in 4 Segmente unterteilt
// jedes Segment kann einen anderen Terrain-Typ besitzen
//--------------------------------------------------------------
typedef struct {
  uint8_t upper_left;   // terrain typ (linke obere ecke)
  uint8_t upper_right;  // terrain typ (rechte obere ecke)
  uint8_t lower_left;   // terrain typ (linke untere ecke)
  uint8_t lower_right;  // terrain typ (rechte untere ecke)
}TILED_TERRAIN_t;
TILED_TERRAIN_t TILED_TERRAIN[TILESET_MAP_ANZ];





//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void UB_Tiled_Map_Init(bool check_map);
void UB_Tiled_Map_Draw(void);
TILED_DIR_t UB_Tiled_Map_Move(TILED_DIR_t direction, uint32_t distance);
void UB_Tiled_Map_Position(uint32_t xp, uint32_t yp);
uint32_t UB_Tiled_Map_GetTileAdr(int32_t xp, int32_t yp);
uint8_t UB_Tiled_Map_GetTerrain(int32_t xp, int32_t yp);



//--------------------------------------------------------------
#endif // __STM32F4_UB_TILED_MAP_H
