//--------------------------------------------------------------
// File     : stm32_ub_tiled_obj.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_TILED_OBJ_H
#define __STM32F4_UB_TILED_OBJ_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32_ub_tiled_global.h"
#include "stm32_ub_tiled_map.h"



//--------------------------------------------------------------
// Aufzaehlung der verschiedenen Objekt-Typen (0...n) [USER_O07]
//--------------------------------------------------------------
#define   TILED_OBJ_KNIGHT1   0  // Knight1
#define   TILED_OBJ_DEVIL1    1  // Devil1
#define   TILED_OBJ_MAGE1     2  // Mage1
#define   TILED_OBJ_MONSTER1  3  // Monster1
#define   TILED_OBJ_COIN      4  // Coin
#define   TILED_OBJ_POISON1   5  // Poison1
#define   TILED_OBJ_POISON2   6  // Poison2
#define   TILED_OBJ_TYP_ANZ   7  // Anzahl der verschiedenen Objekt-Typen


//--------------------------------------------------------------
// Images der Tileset aller Objekt-Typen [USER_O08]
//--------------------------------------------------------------
extern UB_Image Image_Knight1;      // Image vom Tileset (auch C-File einbinden)
extern UB_Image Image_Devil1;       // Image vom Tileset (auch C-File einbinden)
extern UB_Image Image_Mage1;        // Image vom Tileset (auch C-File einbinden)
extern UB_Image Image_Monster1;     // Image vom Tileset (auch C-File einbinden)
extern UB_Image Image_Items1;       // Image vom Tileset (auch C-File einbinden)


//--------------------------------------------------------------
// Anzahl aller Objekte (1...n) [USER_O09]
//--------------------------------------------------------------
#define   TILED_OBJ_ANZ       18  // summe aller Objekte



//--------------------------------------------------------------
// USER : ab hier keine Aenderungen notwendig
//--------------------------------------------------------------



//--------------------------------------------------------------
// Sprites von allen Objekten
//--------------------------------------------------------------
extern TILED_SPRITE_t   SPRITE_OBJ[];


//--------------------------------------------------------------
// Tilesets von allen Objekt-Typen
//--------------------------------------------------------------
extern const TILED_TILESET_t  TILESET_OBJ[];


//--------------------------------------------------------------
// Collision-Typ von allen Objekt-Typen
//--------------------------------------------------------------
typedef enum {
  OBJ_NEUTRAL =0,   // objekt-typ neutral
  OBJ_DEADLY,       // objekt-typ toedlich
  OBJ_COLLECTABLE   // objekt-typ sammelbar
}COLLISION_TYP_t;
extern const COLLISION_TYP_t COLLISION_OBJ[];


//--------------------------------------------------------------
// Struktur von einem Wegpunkt
//
// dient zum bewegen von Objekten
// damit koennen feste wegrouten realisiert werden
// Das objekt bewegt sich von wegpunkt zu wegpunkt
//--------------------------------------------------------------
typedef struct {
  uint32_t nr;            // nr vom wegpunkt
  TILED_DIR_t direction;  // richtung zum bewegen
  uint32_t distance;      // distanz (in pixel) zum bewegen
  uint32_t speed;         // schrittweite (in pixel)
  uint32_t next;          // nr vom naechsten wegpunkt
}TILED_WAYPOINT_t;


//--------------------------------------------------------------
// Struktur von einem Objekt
//--------------------------------------------------------------
typedef struct {
  uint8_t typ;            // typ vom objekt
  bool enable;            // objekt enable/disable
  uint32_t waypoint;      // aktueller wegpunkt zum bewegen
  uint32_t distance;      // restliche distanz (wird berechnet)
}TILED_OBJ_t;
extern TILED_OBJ_t TILED_OBJ[];


//--------------------------------------------------------------
// Animation von allen Objekten
//--------------------------------------------------------------
TILED_ANIMATION_t  ANIMATION_OBJ[TILED_OBJ_ANZ];




//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void UB_Tiled_Obj_Init(void);
void UB_Tiled_Obj_Draw(void);
void UB_Tiled_Obj_MoveAll(void);


//--------------------------------------------------------------
#endif // __STM32F4_UB_TILED_PLAYER_H
